@@ -39,6 +39,7 @@ import java.security.cert.CertificateException;
 import java.security.GeneralSecurityException;
 import org.mozilla.jss.pkcs11.PK11Cert;
 import java.util.*;
+import org.mozilla.jss.pkcs11.KeyType;
 import org.mozilla.jss.pkcs11.PK11Token;
 import org.mozilla.jss.pkcs11.PK11Module;
 import org.mozilla.jss.pkcs11.PK11SecureRandom;
@@ -52,7 +53,7 @@ import org.mozilla.jss.provider.java.security.JSSMessageDigestSpi;
  * Initialization is done with static methods, and must be done before
  * an instance can be created.  All other operations are done with instance
  * methods.
- * @version $Revision: 1.15 $ $Date: 2002/12/13 06:12:28 $
+ * @version $Revision: 1.20 $ $Date: 2003/08/27 00:08:41 $
  */
 public final class CryptoManager implements TokenSupplier
 {
@@ -747,9 +748,7 @@ public final class CryptoManager implements TokenSupplier
 
     /**
      * This function sets the global password callback.  It is
-     * not thread-safe to change this. A better strategy than using
-     * callbacks is to explicitly login to the tokens you need to use.
-     * Password callbacks are then only used as a last resort.
+     * not thread-safe to change this.
      * <p>The callback may be NULL, in which case password callbacks will
      * fail gracefully.
      */
@@ -855,18 +854,26 @@ public final class CryptoManager implements TokenSupplier
                 instance.reloadModules();
             }
         }
-        if( values.installJSSProvider ) {
-            // Force class load before we install the provider. Otherwise we get
-            // an infinite loop as the Security manager tries to instantiate the
-            // digest to verify its own JAR file.
-            JSSMessageDigestSpi mds = new JSSMessageDigestSpi.SHA1();
 
+        // Force class load before we install the provider. Otherwise we get
+        // an infinite loop as the Security manager tries to instantiate the
+        // digest to verify its own JAR file.
+        JSSMessageDigestSpi mds = new JSSMessageDigestSpi.SHA1();
+        // Force the KeyType class to load before we can install JSS as a
+        // provider.  JSS's signature provider accesses KeyType.
+        KeyType kt = KeyType.getKeyTypeFromAlgorithm(
+            SignatureAlgorithm.RSASignatureWithSHA1Digest);
+
+        if( values.installJSSProvider ) {
             int position = java.security.Security.insertProviderAt(
                             new JSSProvider(), 1);
-            if(position==-1) {
-                Debug.trace(Debug.ERROR,
-                    "Unable to install default provider");
-            }
+            // This returns -1 if the provider was already installed, in which
+            // case it is not installed again.  Is this
+            // an error? I don't think so, although it might be confusing
+            // if the provider is not in the position they expected.
+            // However, this will only happen if they are installing the
+            // provider themselves, so presumably they know what they're
+            // doing.
         }
         if( values.removeSunProvider ) {
             java.security.Security.removeProvider("SUN");
@@ -1302,11 +1309,11 @@ public final class CryptoManager implements TokenSupplier
     /********************************************************************/
 
     public static final String
-    JAR_JSS_VERSION     = "JSS_VERSION = JSS_3_3";
+    JAR_JSS_VERSION     = "JSS_VERSION = JSS_3_4_1";
     public static final String
     JAR_JDK_VERSION     = "JDK_VERSION = N/A";
     public static final String
-    JAR_NSS_VERSION     = "NSS_VERSION = NSS_3_6_1_RTM";
+    JAR_NSS_VERSION     = "NSS_VERSION = NSS_3_7_8_RTM";
     public static final String
     JAR_DBM_VERSION     = "DBM_VERSION = N/A";
     public static final String
