@@ -54,11 +54,13 @@ public final class JSSProvider extends java.security.Provider {
 
         put("Signature.MD5/RSA",
             "org.mozilla.jss.provider.java.security.JSSSignatureSpi$MD5RSA");
+        put("Alg.Alias.Signature.MD5withRSA", "MD5/RSA");
+
         put("Signature.MD2/RSA",
             "org.mozilla.jss.provider.java.security.JSSSignatureSpi$MD2RSA");
+
         put("Signature.SHA-1/RSA",
             "org.mozilla.jss.provider.java.security.JSSSignatureSpi$SHA1RSA");
-
         put("Alg.Alias.Signature.SHA1/RSA", "SHA-1/RSA");
         put("Alg.Alias.Signature.SHA1withRSA", "SHA-1/RSA");
 
@@ -118,6 +120,8 @@ public final class JSSProvider extends java.security.Provider {
             "org.mozilla.jss.provider.javax.crypto.JSSCipherSpi$RC4");
         put("Cipher.RSA",
             "org.mozilla.jss.provider.javax.crypto.JSSCipherSpi$RSA");
+        put("Cipher.RC2",
+            "org.mozilla.jss.provider.javax.crypto.JSSCipherSpi$RC2");
 
         /////////////////////////////////////////////////////////////
         // KeyGenerator
@@ -131,6 +135,8 @@ public final class JSSProvider extends java.security.Provider {
             "org.mozilla.jss.provider.javax.crypto.JSSKeyGeneratorSpi$AES");
         put("KeyGenerator.RC4",
             "org.mozilla.jss.provider.javax.crypto.JSSKeyGeneratorSpi$RC4");
+        put("KeyGenerator.RC2",
+            "org.mozilla.jss.provider.javax.crypto.JSSKeyGeneratorSpi$RC2");
         put("KeyGenerator.HmacSHA1",
            "org.mozilla.jss.provider.javax.crypto.JSSKeyGeneratorSpi$HmacSHA1");
         put("KeyGenerator.PBAHmacSHA1",
@@ -148,6 +154,8 @@ public final class JSSProvider extends java.security.Provider {
             "org.mozilla.jss.provider.javax.crypto.JSSSecretKeyFactorySpi$AES");
         put("SecretKeyFactory.RC4",
             "org.mozilla.jss.provider.javax.crypto.JSSSecretKeyFactorySpi$RC4");
+        put("SecretKeyFactory.RC2",
+            "org.mozilla.jss.provider.javax.crypto.JSSSecretKeyFactorySpi$RC2");
         put("SecretKeyFactory.HmacSHA1",
             "org.mozilla.jss.provider.javax.crypto.JSSSecretKeyFactorySpi$HmacSHA1");
         put("SecretKeyFactory.PBAHmacSHA1",
