@@ -377,9 +377,14 @@ public static class Template implements ASN1Template {
                     // use the default
                     seq.addElement( e.getDefault() );
                 } else {
+                    String tagDesc;
+                    if( lookAhead == null ) {
+                        tagDesc = "(null)";
+                    } else {
+                        tagDesc = lookAhead.getTag().toString();
+                    }
                     throw new InvalidBERException("Missing item #" + index +
-                        ": expecting " + lookAhead.getTag() + ", found"
-                        + e.getImplicitTag() );
+                        ": found " + tagDesc );
                 }
                 continue;
             }
@@ -407,12 +412,14 @@ public static class Template implements ASN1Template {
             }
 
             // Store this element in the SEQUENCE
-            if(  e.getImplicitTag() == null ) {
-                // no implicit tag
-                seq.addElement( val );
-            } else {
-                // there is an implicit tag
-                seq.addElement( e.getImplicitTag(), val );
+            if( e.producesOutput() ) {
+                if(  e.getImplicitTag() == null ) {
+                    // no implicit tag
+                    seq.addElement( val );
+                } else {
+                    // there is an implicit tag
+                    seq.addElement( e.getImplicitTag(), val );
+                }
             }
 
             // If this element is repeatable, don't go on to the next element
@@ -444,7 +451,8 @@ public static class Template implements ASN1Template {
         return seq;
 
       } catch(InvalidBERException e) {
-        throw new InvalidBERException(e, "SEQUENCE(item #"+index+")");
+        e.append("SEQUENCE(item #" +index + ")");
+        throw e;
       }
     }
 
@@ -458,11 +466,21 @@ public static class Template implements ASN1Template {
          * Creates a new element, which may or may not be optional.
          */
         public Element(Tag implicitTag, ASN1Template type, boolean optional)
+        {
+            this(implicitTag, type, optional, true);
+        }
+
+        /**
+         * Creates a new element, which may or may not be optional.
+         */
+        public Element(Tag implicitTag, ASN1Template type, boolean optional,
+            boolean doesProduceOutput)
         {
             this.type = type;
             defaultVal = null;
             this.optional = optional;
             this.implicitTag = implicitTag;
+            this.doesProduceOutput = doesProduceOutput;
         }
 
         /**
@@ -476,6 +494,11 @@ public static class Template implements ASN1Template {
             this.implicitTag = implicitTag;
         }
 
+        private boolean doesProduceOutput = true;
+        boolean producesOutput() {
+            return doesProduceOutput;
+        }
+
         // repeatability is provided to allow for SEQUENCE OF SIZE
         // constructs.  It is package private.
         private boolean repeatable;
@@ -548,6 +571,15 @@ public static class OF_Template implements ASN1Template {
         template.addElement( el );
     }
 
+    public static OF_Template makeOutputlessOFTemplate(ASN1Template type) {
+        OF_Template t = new OF_Template();
+        t.template = new Template();
+        Template.Element el = new Template.Element(null, type, true, false);
+        el.makeRepeatable();
+        t.template.addElement(el);
+        return t;
+    }
+
     public boolean tagMatch(Tag tag) {
         return TAG.equals(tag);
     }
