@@ -95,14 +95,10 @@ final class PK11Cipher extends org.mozilla.jss.crypto.Cipher {
         byte[] IV = null;
         if( params instanceof IVParameterSpec ) {
             IV = ((IVParameterSpec)params).getIV();
-        }
-        try {
-            if( params instanceof IvParameterSpec ) {
-                IV = ((IvParameterSpec)params).getIV();
-            }
-        } catch(NoClassDefFoundError e) {
-            // javax.crypto.spec.IvParameterSpec was introduced in JDK 1.4.
-            // Older versions of the JRE don't have it.
+        } else if( params instanceof IvParameterSpec ) {
+            IV = ((IvParameterSpec)params).getIV();
+        } else if( params instanceof RC2ParameterSpec ) {
+            IV = ((RC2ParameterSpec)params).getIV();
         }
         return IV;
     }
@@ -122,7 +118,12 @@ final class PK11Cipher extends org.mozilla.jss.crypto.Cipher {
         this.parameters = parameters;
         state = ENCRYPT;
 
-        contextProxy = initContext( true, key, algorithm, IV );
+        if( parameters instanceof RC2ParameterSpec ) {
+            contextProxy = initContextWithKeyBits( true, key, algorithm, IV,
+                ((RC2ParameterSpec)parameters).getEffectiveKeyBits() );
+        } else {
+            contextProxy = initContext( true, key, algorithm, IV );
+        }
     }
 
     public void initDecrypt(SymmetricKey key, AlgorithmParameterSpec parameters)
@@ -139,7 +140,12 @@ final class PK11Cipher extends org.mozilla.jss.crypto.Cipher {
         this.parameters = parameters;
         state = DECRYPT;
 
-        contextProxy = initContext(false, key, algorithm, IV);
+        if( parameters instanceof RC2ParameterSpec ) {
+            contextProxy = initContextWithKeyBits(false, key, algorithm, IV,
+                ((RC2ParameterSpec)parameters).getEffectiveKeyBits() );
+        } else {
+            contextProxy = initContext(false, key, algorithm, IV);
+        }
     }
 
     public byte[] update(byte[] bytes)
@@ -204,10 +210,16 @@ final class PK11Cipher extends org.mozilla.jss.crypto.Cipher {
     }
 
     private static native CipherContextProxy
-    initContext( boolean encrypt, SymmetricKey key, EncryptionAlgorithm alg,
+    initContext(boolean encrypt, SymmetricKey key, EncryptionAlgorithm alg,
                  byte[] IV)
         throws TokenException;
 
+    // This version accepts the number of effective key bits for RC2 CBC.
+    private static native CipherContextProxy
+    initContextWithKeyBits(boolean encrypt, SymmetricKey key,
+                EncryptionAlgorithm alg, byte[] IV, int keyBits)
+        throws TokenException;
+
     private static native byte[]
     updateContext( CipherContextProxy context, byte[] input, int blocksize )
         throws TokenException;
