@@ -42,6 +42,8 @@ import org.mozilla.jss.util.Assert;
 import java.security.PublicKey;
 import java.security.interfaces.RSAPublicKey;
 import java.security.interfaces.DSAPublicKey;
+import javax.crypto.spec.IvParameterSpec;
+import javax.crypto.spec.RC2ParameterSpec;
 
 final class PK11KeyWrapper implements KeyWrapper {
 
@@ -239,13 +241,10 @@ final class PK11KeyWrapper implements KeyWrapper {
         }
         if( params instanceof IVParameterSpec ) {
             IV = ((IVParameterSpec)params).getIV();
-        }
-        try {
-            if( params instanceof javax.crypto.spec.IvParameterSpec ) {
-                IV = ((javax.crypto.spec.IvParameterSpec)params).getIV();
-            }
-        } catch(NoClassDefFoundError e) {
-            // we must be running in JDK < 1.4. Ignore.
+        } else if( params instanceof javax.crypto.spec.IvParameterSpec ) {
+            IV = ((javax.crypto.spec.IvParameterSpec)params).getIV();
+        } else if( params instanceof RC2ParameterSpec ) {
+            IV = ((RC2ParameterSpec)params).getIV();
         }
     }
 
@@ -529,9 +528,11 @@ final class PK11KeyWrapper implements KeyWrapper {
             return EncryptionAlgorithm.DES_ECB;
         } else if( type == SymmetricKey.DES3 ) {
             return EncryptionAlgorithm.DES3_ECB;
-        } else {
-            Assert._assert( type == SymmetricKey.RC4 );
+        } else if( type == SymmetricKey.RC4 ) {
             return EncryptionAlgorithm.RC4;
+        } else {
+            Assert._assert( type == SymmetricKey.RC2 );
+            return EncryptionAlgorithm.RC2_CBC;
         }
     }
 
