@@ -220,7 +220,7 @@ traverseTokenObjects
 
         certList = PK11_ListCertsInSlot(slot);
         if( certList == NULL ) {
-            JSS_throwMsg(env, TOKEN_EXCEPTION,
+            JSS_throwMsgPrErr(env, TOKEN_EXCEPTION,
                 "Failed to list certificates on token");
             goto finish;
         }
@@ -503,14 +503,13 @@ engineGetCertificateTraversalCallback
     return travStat;
 }
         
-
-JNIEXPORT jbyteArray JNICALL
-Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_getDERCert
-    (JNIEnv *env, jobject this, jstring alias)
+static CERTCertificate*
+lookupCertByNickname(JNIEnv *env, jobject this, jstring alias)
 {
     PK11SlotInfo *slot;
     EngineGetCertificateCBInfo cbinfo = {NULL,NULL};
     jbyteArray derCertBA = NULL;
+    PRStatus status = PR_FAILURE;
 
     if( alias == NULL ) goto finish;
 
@@ -522,26 +521,67 @@ Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_getDERCert
     cbinfo.targetNickname = (*env)->GetStringUTFChars(env, alias, NULL);
     if(cbinfo.targetNickname == NULL ) goto finish;
 
-    if( traverseTokenObjects(   env,
-                                slot,
-                                engineGetCertificateTraversalCallback,
-                                CERT,
-                                (void*) &cbinfo)
-            != PR_SUCCESS )
-    {
-        goto finish;
-    }
-
-    if( cbinfo.cert != NULL ) {
-        derCertBA = JSS_SECItemToByteArray(env, &cbinfo.cert->derCert);
-    }
+    status  = traverseTokenObjects( env,
+                                    slot,
+                                    engineGetCertificateTraversalCallback,
+                                    CERT,
+                                    (void*) &cbinfo);
 
 finish:
     if( cbinfo.targetNickname != NULL ) {
         (*env)->ReleaseStringUTFChars(env, alias, cbinfo.targetNickname);
     }
-    if( cbinfo.cert != NULL) {
+    if( status != PR_SUCCESS && cbinfo.cert != NULL) {
         CERT_DestroyCertificate(cbinfo.cert);
+        cbinfo.cert = NULL;
+    }
+    return cbinfo.cert;
+}
+
+JNIEXPORT jobject JNICALL
+Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_getCertObject
+    (JNIEnv *env, jobject this, jstring alias)
+{
+    CERTCertificate *cert = NULL;
+    jobject certObj = NULL;
+
+    cert = lookupCertByNickname(env, this, alias);
+    if( cert == NULL ) {
+        goto finish;
+    }
+
+    certObj = JSS_PK11_wrapCert(env, &cert);
+
+finish:
+    if( cert != NULL ) {
+        CERT_DestroyCertificate(cert);
+    }
+    if( certObj == NULL ) {
+        PR_ASSERT( (*env)->ExceptionOccurred(env) );
+    }
+    return certObj;
+}
+
+JNIEXPORT jbyteArray JNICALL
+Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_getDERCert
+    (JNIEnv *env, jobject this, jstring alias)
+{
+    CERTCertificate * cert = NULL;
+    jbyteArray derCertBA = NULL;
+
+    cert = lookupCertByNickname(env, this, alias);
+    if( cert == NULL ) {
+        goto finish;
+    }
+
+    derCertBA = JSS_SECItemToByteArray(env, &cert->derCert);
+
+finish:
+    if( cert != NULL) {
+        CERT_DestroyCertificate(cert);
+    }
+    if( derCertBA == NULL ) {
+        PR_ASSERT( (*env)->ExceptionOccurred(env) );
     }
     return derCertBA;
 }
@@ -660,10 +700,23 @@ getKeyByCertNickCallback
     return travStat;
 }
         
-
+/*
+ * This is only here for backward compatibility.
+ */
 JNIEXPORT jobject JNICALL
 Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engineGetKey
     (JNIEnv *env, jobject this, jstring alias, jcharArray password)
+{
+    PR_ASSERT(0);
+    JSS_throwMsg(env, "java/lang/RuntimeException", 
+        "Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engine"
+        "GetKey not implemented");
+    return NULL;
+}
+
+JNIEXPORT jobject JNICALL
+Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engineGetKeyNative
+    (JNIEnv *env, jobject this, jstring alias, jcharArray password)
 {
     PK11SlotInfo *slot=NULL;
     FindKeyCBInfo keyCbinfo = {NULL, NULL, NULL};
@@ -830,29 +883,23 @@ Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engineSetKeyEntryNati
         goto finish;
     }
 
-#if 1
-    JSS_throwMsg(env, KEYSTORE_EXCEPTION,
-        "Operation is not supported. " 
-        "http://bugzilla.mozilla.org/show_bug.cgi?id=128172");
-    goto finish;
-#else
     if( (*env)->IsInstanceOf(env, keyObj, privkClass) ) {
         SECKEYPrivateKey *privk;
 
         if( JSS_PK11_getPrivKeyPtr(env, keyObj, &privk) != PR_SUCCESS ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
                 "Failed to extract NSS key from private key object");
             goto finish;
         }
 
-        tokenPrivk = PK11_ConvertSessionPrivKeyToTokenPrivKey(privk);
+        tokenPrivk = PK11_ConvertSessionPrivKeyToTokenPrivKey(privk, NULL);
         if( tokenPrivk == NULL ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
                 "Failed to copy private key to permanent token object");
             goto finish;
         }
         if( PK11_SetPrivateKeyNickname(tokenPrivk, nickname) != SECSuccess ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
                 "Failed to set alias of copied private key");
             goto finish;
         }
@@ -860,20 +907,20 @@ Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engineSetKeyEntryNati
         PK11SymKey *symk;
 
         if( JSS_PK11_getSymKeyPtr(env, keyObj, &symk) != PR_SUCCESS ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
                 "Failed to extract NSS key from symmetric key object");
             goto finish;
         }
 
-        if( PK11_SetSymKeyNickname(symk, nickname) != SECSuccess ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
-                "Failed to set alias of symmetric key");
+        tokenSymk = PK11_ConvertSessionSymKeyToTokenSymKey(symk, NULL);
+        if( tokenSymk == NULL ) {
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
+                "Failed to copy symmetric key to permanent token object");
             goto finish;
         }
-        tokenSymk = PK11_ConvertSessionSymKeyToTokenSymKey(symk);
-        if( tokenSymk == NULL ) {
-            JSS_throwMsg(env, KEYSTORE_EXCEPTION,
-                "Failed to copy symmetric key to permanent token object");
+        if( PK11_SetSymKeyNickname(tokenSymk, nickname) != SECSuccess ) {
+            JSS_throwMsgPrErr(env, KEYSTORE_EXCEPTION,
+                "Failed to set alias of symmetric key");
             goto finish;
         }
     } else {
@@ -882,7 +929,6 @@ Java_org_mozilla_jss_provider_java_security_JSSKeyStoreSpi_engineSetKeyEntryNati
             " JSS symmetric key (PK11SymKey)");
         goto finish;
     }
-#endif
 
 finish:
     if( nickname != NULL ) {
